@@ -4,7 +4,7 @@
 ! 
 ! USAGE:
 ! 
-! call loglassoNET (lam2, nobs, nvars, x, y, jd, pf, dfmax, &
+! call loglassoNET (lam2, nobs, nvars, x, y, jd, pf, pf2, dfmax, &
 ! & pmax, nlam, flmin, ulam, eps, isd, maxit, nalam, b0, beta, ibeta, &
 ! & nbeta, alam, npass, jerr)
 ! 
@@ -19,8 +19,10 @@
 !    jd(jd(1)+1) = predictor variable deletion flag
 !                  jd(1) = 0  => use all variables
 !                  jd(1) != 0 => do not use variables jd(2)...jd(jd(1)+1)
-!    pf(nvars) = relative penalties for each predictor variable
+!    pf(nvars) = relative L1 penalties for each predictor variable
 !                pf(j) = 0 => jth variable unpenalized
+!    pf2(nvars) = relative L2 penalties for each predictor variable
+!                pf2(j) = 0 => jth variable unpenalized
 !    dfmax = limit the maximum number of variables in the model.
 !            (one of the stopping criterion)
 !    pmax = limit the maximum number of variables ever to be nonzero. 
@@ -77,7 +79,7 @@
 
 
 ! --------------------------------------------------
-SUBROUTINE loglassoNET (lam2, nobs, nvars, x, y, jd, pf, dfmax, pmax, &
+SUBROUTINE loglassoNET (lam2, nobs, nvars, x, y, jd, pf, pf2, dfmax, pmax, &
 & nlam, flmin, ulam, eps, isd, maxit, nalam, b0, beta, ibeta, nbeta, &
 & alam, npass, jerr)
 ! --------------------------------------------------
@@ -102,6 +104,7 @@ SUBROUTINE loglassoNET (lam2, nobs, nvars, x, y, jd, pf, dfmax, pmax, &
       DOUBLE PRECISION :: x (nobs, nvars)
       DOUBLE PRECISION :: y (nobs)
       DOUBLE PRECISION :: pf (nvars)
+      DOUBLE PRECISION :: pf2 (nvars)
       DOUBLE PRECISION :: ulam (nlam)
       DOUBLE PRECISION :: beta (pmax, nlam)
       DOUBLE PRECISION :: b0 (nlam)
@@ -136,10 +139,14 @@ SUBROUTINE loglassoNET (lam2, nobs, nvars, x, y, jd, pf, dfmax, pmax, &
          jerr = 10000
          RETURN
       END IF
+      IF (maxval(pf2) <= 0.0D0) THEN
+         jerr = 10000
+         RETURN
+      END IF
       pf = Max (0.0D0, pf)
-      pf = pf * nvars / sum (pf)
+      pf2 = Max (0.0D0, pf2)
       CALL standard (nobs, nvars, x, ju, isd, xmean, xnorm, maj)
-      CALL loglassoNETpath (lam2, maj, nobs, nvars, x, y, ju, pf, &
+      CALL loglassoNETpath (lam2, maj, nobs, nvars, x, y, ju, pf, pf2, &
      & dfmax, pmax, nlam, flmin, ulam, eps, maxit, nalam, b0, beta, &
      & ibeta, nbeta, alam, npass, jerr)
       IF (jerr > 0) RETURN! check error after calling function
@@ -158,7 +165,7 @@ SUBROUTINE loglassoNET (lam2, nobs, nvars, x, y, jd, pf, dfmax, pmax, &
       RETURN
 END SUBROUTINE loglassoNET
 ! --------------------------------------------------
-SUBROUTINE loglassoNETpath (lam2, maj, nobs, nvars, x, y, ju, pf, &
+SUBROUTINE loglassoNETpath (lam2, maj, nobs, nvars, x, y, ju, pf, pf2, &
 & dfmax, pmax, nlam, flmin, ulam, eps, maxit, nalam, b0, beta, m, &
 & nbeta, alam, npass, jerr)
 ! --------------------------------------------------
@@ -185,6 +192,7 @@ SUBROUTINE loglassoNETpath (lam2, maj, nobs, nvars, x, y, ju, pf, &
       DOUBLE PRECISION :: x (nobs, nvars)
       DOUBLE PRECISION :: y (nobs)
       DOUBLE PRECISION :: pf (nvars)
+      DOUBLE PRECISION :: pf2 (nvars)
       DOUBLE PRECISION :: beta (pmax, nlam)
       DOUBLE PRECISION :: ulam (nlam)
       DOUBLE PRECISION :: b0 (nlam)
@@ -270,12 +278,12 @@ SUBROUTINE loglassoNETpath (lam2, maj, nobs, nvars, x, y, ju, pf, &
                DO k = 1, nvars
                   IF (ju(k) /= 0) THEN
                      oldb = b (k)
-                     u = dot_product (y/(1.0D0+Exp(r)), x(:, k))
+                     u = dot_product (y/(1.0D0 + Exp(r)), x(:, k))
                      u = maj (k) * b (k) + u / nobs
                      v = al * pf (k)
                      v = Abs (u) - v
                      IF (v > 0.0D0) THEN
-                        b (k) = sign (v, u) / (maj(k)+lam2)
+                        b (k) = sign (v, u) / (maj(k) + pf2(k) * lam2)
                      ELSE
                         b (k) = 0.0D0
                      END IF
@@ -308,12 +316,12 @@ SUBROUTINE loglassoNETpath (lam2, maj, nobs, nvars, x, y, ju, pf, &
                   DO j = 1, ni
                      k = m (j)
                      oldb = b (k)
-                     u = dot_product (y/(1.0D0+Exp(r)), x(:, k))
+                     u = dot_product (y/(1.0D0 + Exp(r)), x(:, k))
                      u = maj (k) * b (k) + u / nobs
                      v = al * pf (k)
                      v = Abs (u) - v
                      IF (v > 0.0D0) THEN
-                        b (k) = sign (v, u) / (maj(k)+lam2)
+                        b (k) = sign (v, u) / (maj(k) + pf2(k) * lam2)
                      ELSE
                         b (k) = 0.0D0
                      END IF
